@@ -1,10 +1,10 @@
 package YATT::Lite; sub MY () {__PACKAGE__}
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 use 5.010; no if $] >= 5.017011, warnings => "experimental";
 
 use Carp qw(carp croak confess);
-our $VERSION = '0.100';
+our $VERSION = '0.101';
 #use mro 'c3';
 
 use Scalar::Util qw/weaken/;
@@ -17,6 +17,7 @@ use YATT::Lite::MFields qw/YATT
 	      cf_dir
 	      cf_vfs cf_base
 	      cf_factory
+	      cf_header_charset
 	      cf_output_encoding
 	      cf_tmpl_encoding
 	      cf_index_name
@@ -32,8 +33,11 @@ use YATT::Lite::MFields qw/YATT
 	      cf_dont_debug_param
 	      cf_info
 	      cf_lcmsg_sink
+	      cf_always_refresh_deps
 
 	      cf_default_lang
+
+	      cf_path2entns
 	    /;
 
 MY->cf_mkaccessors(qw/app_name/);
@@ -102,7 +106,7 @@ sub handle {
       ." in $self->{cf_app_ns}.\n";
   }
 
-  my $sub = $YATT->find_handler($ext, $file);
+  my $sub = $YATT->find_handler($ext, $file, $CON);
   $sub->($YATT, $CON, $file);
 
   try_invoke($CON, 'flush_headers');
@@ -128,7 +132,7 @@ sub render_into {
 }
 
 sub find_handler {
-  (my MY $self, my ($ext, $file)) = @_;
+  (my MY $self, my ($ext, $file, $con)) = @_;
   $ext //= $self->cut_ext($file) || $self->{cf_ext_public};
   $ext = "yatt" if $ext eq $self->{cf_ext_public};
   my $sub = $self->can("_handle_$ext")
@@ -257,6 +261,7 @@ sub fconfigure_encoding {
 # Delayed loading of YATT::Lite::Core
 #========================================
 
+*open_vfs = *open_trans; *open_vfs = *open_trans;
 sub open_trans {
   (my MY $self) = @_;
   my $trans = $self->get_trans;
@@ -264,6 +269,7 @@ sub open_trans {
   $trans;
 }
 
+*get_vfs = *get_trans; *get_vfs = *get_trans;
 sub get_trans {
   (my MY $self) = @_;
   $self->{YATT} || $self->build_trans($self->{cf_tmpl_cache});
@@ -293,7 +299,9 @@ sub build_trans {
 				     ext_private
 				     rc_script
 				     lcmsg_sink
-				     only_parse/));
+				     only_parse
+				     always_refresh_deps
+				    /));
 }
 
 sub _before_after_new {
