@@ -1,13 +1,10 @@
 package YATT::Lite::CGen::Perl;
 use strict;
 use warnings qw(FATAL all NONFATAL misc);
-use mro 'c3';
 
 require 5.010; # For named capture.
 
-use constant DEBUG_MRO => $ENV{DEBUG_YATT_MRO};
-
-use YATT::Lite::Core qw(Folder Template Part Widget Action Entity);
+use YATT::Lite::Core qw(Folder Template Part Widget Action);
 use YATT::Lite::Constants;
 
 # Naming convention:
@@ -20,8 +17,7 @@ use YATT::Lite::Constants;
   #========================================
   package YATT::Lite::CGen::Perl; sub MY () {__PACKAGE__}
   use base qw(YATT::Lite::CGen);
-  use YATT::Lite::MFields;
-  use YATT::Lite::Util qw(lexpand numLines globref terse_dump catch);
+  use YATT::Lite::Util qw(lexpand numLines globref terse_dump);
   use Carp;
   #========================================
   sub list_inheritance {
@@ -40,8 +36,8 @@ use YATT::Lite::Constants;
       }
     } $tmpl->list_base
   }
-  sub setup_inheritance_for {
-    (my MY $self, my $spec, my Template $tmpl) = @_;
+  sub setup_inheritance {
+    (my MY $self, my Template $tmpl) = @_;
     unless (defined $tmpl->{cf_entns}) {
       die "BUG: EntNS is empty for '$tmpl->{cf_name}'!";
     }
@@ -50,28 +46,15 @@ use YATT::Lite::Constants;
     unless (defined $glob) {
       die "BUG: ISA glob for '$tmpl->{cf_name}' is empty!";
     }
-    $self->ensure_generated_for_folders($spec, $tmpl->list_base);
     my @isa = $self->list_inheritance($tmpl);
     if (grep {not defined} @isa) {
       die "BUG: ISA for '$tmpl->{cf_name}' contains undef!";
     }
-    if (my $err = catch {
-      *$glob = \@isa;
-    }) {
-      die $self->generror("Can't set ISA for '%s' as [%s]: %s"
-			  , $tmpl->{cf_name}
-			  , join(", ", @isa)
-			  , $err
-			);
-    }
+    *$glob = \@isa;
   }
   sub generate_inheritance {
     (my MY $self, my Template $tmpl) = @_;
-    my @isa = $self->list_inheritance($tmpl);
-    my $mro = mro::get_mro($tmpl->{cf_entns});
-    print STDERR "($mro) [$tmpl->{cf_path}] $tmpl->{cf_entns}::ISA = @isa\n"
-      if DEBUG_MRO;
-    sprintf q{use mro '%s'; our @ISA = qw(%s); }, $mro, join " ", @isa;
+    sprintf q{our @ISA = qw(%s); }, join " ", $self->list_inheritance($tmpl);
   }
   #========================================
   sub generate_preamble {
@@ -101,8 +84,7 @@ use YATT::Lite::Constants;
   sub generate_widget {
     (my MY $self, my Widget $widget, my ($widget_name, $tmpl_path)) = @_;
     if ($widget->{cf_suppressed}) {
-      # First line is alread used for package declaration.
-      return "\n" x ((($widget->{cf_endln} - 1) - $widget->{cf_startln}) - 2);
+      return "\n" x ($widget->{cf_endln} - $widget->{cf_startln});
     }
     break_cgen();
     local $self->{curwidget} = $widget;
@@ -124,51 +106,14 @@ use YATT::Lite::Constants;
   sub generate_action {
     (my MY $self, my Action $action) = @_;
     # XXX: 改行の調整が必要。
-    my @src = ($self->sync_curline($action->{cf_startln})
-               , "sub do_$$action{cf_name} {");
     my $src = $self->{curtmpl}->source_substr
       ($action->{cf_bodypos}, $action->{cf_bodylen});
-
-    if (lexpand($action->{arg_order})
-        or $src !~ m{^([\ \t\r\n]*)my\s*\([^;\)]+\)\s*=\s*\@_\s*;}) {
-      # If an action has no arguments
-      # and its source doesn't start with my (...) = @_;,
-      # insert preamble and getargs.
-      push @src, $self->gen_preamble($action)
-        , $self->gen_getargs($action, not $action->{cf_implicit});
-    }
-
     my $has_nl = $src =~ s/\r?\n\Z//;
     $self->{curline} = $action->{cf_bodyln} + numLines($src)
       + ($has_nl ? 1 : 0);
-    (@src, $src, "}");
+    sprintf "sub %s {%s}\n"
+      , $action->{cf_name}, $src;
   }
-
-  # XXX: As you see, dup code.
-  # XXX: Should define common base class for Action and Entity.
-  sub generate_entity {
-    (my MY $self, my Entity $entity) = @_;
-    # XXX: 改行の調整が必要。
-    my @src = ($self->sync_curline($entity->{cf_startln})
-               , "sub entity_$$entity{cf_name} {");
-    my $src = $self->{curtmpl}->source_substr
-      ($entity->{cf_bodypos}, $entity->{cf_bodylen});
-
-    if (lexpand($entity->{arg_order})
-        or $src !~ m{^([\ \t\r\n]*)my\s*\([^;\)]+\)\s*=\s*\@_\s*;}) {
-      # If an entity has no arguments
-      # and its source doesn't start with my (...) = @_;,
-      # insert preamble and getargs.
-      push @src, q{ my ($this) = shift; my $CON = $this->CON;}
-        , $self->gen_getargs($entity, not $entity->{cf_implicit});
-    }
-
-    my $has_nl = $src =~ s/\r?\n\Z//;
-    $self->{curline} = $entity->{cf_bodyln} + numLines($src)
-      + ($has_nl ? 1 : 0);
-    (@src, $src, "}");
-  }
-
   #========================================
   sub gen_preamble {q{ my ($this, $CON) = splice @_, 0, 2;}}
   sub gen_getargs {
@@ -244,13 +189,6 @@ use YATT::Lite::Constants;
     while (@{$self->{curtoks}}) {
       my $node = shift @{$self->{curtoks}};
       unless (ref $node) {
-        # 次が element な時は、先行する indent を捨てる
-        if (@{$self->{curtoks}} and ref $self->{curtoks}[0]
-              and $self->{curtoks}[0][0] == TYPE_ELEMENT
-              and $node =~ /^[\ \t]+\z/
-          ) {
-          next;
-        }
 	# text node の末尾が改行で終わっている場合、 明示的に "\n" を生成する
 	my $has_nl = $node =~ s/\r?\n\Z//s;
 	push @queue, qtext($node) if $node ne ''; # 削ったら空になるかも。
@@ -276,11 +214,6 @@ use YATT::Lite::Constants;
 	push @queue, $expr;
 	$flush->() if $expr =~ /\n/;
       }
-      # Following can remove trailing newline from </yatt:foreach>,
-      # but it affects all of yatt tags.
-      # if (@{$self->{curtoks}} and $self->{curtoks}[0] eq "\n") {
-      #   shift @{$self->{curtoks}};
-      # }
     }
     $flush->();
     join " ", @result;
@@ -302,10 +235,9 @@ use YATT::Lite::Constants;
 	next;
       }
       # 許されるのは entity だけでは？ でもないか。 element 引数の時は、capture したいはず。
-      my $sub = $dispatch->[$node->[0]] or do {
-        die $self->generror("gen_as %s: Unknown node type: %d"
-                            , $type, $node->[0]);
-      };
+      my $sub = $dispatch->[$node->[0]]
+	or die $self->generror("gen_as %s: Unknown node type: %d"
+			       , $type, $node->[0]);
       my $expr = $sub->($self, $node);
       next unless defined $expr;
       if (ref $expr) {
@@ -356,13 +288,9 @@ use YATT::Lite::Constants;
   }
   sub text_from_element {
     (my MY $self, my $node) = @_;
+    &YATT::Lite::Breakpoint::breakpoint();
     my $call_ref = $self->from_element($node);
-    sprintf q{(YATT::Lite::Util::captured {my ($CON) = @_; %s})}, $$call_ref;
-  }
-  sub text_from_pi {
-    (my MY $self, my $node) = @_;
-    my $statements = $self->from_pi($node);
-    sprintf q{(YATT::Lite::Util::captured {my ($CON) = @_; %s})}, $statements;
+    sprintf q{YATT::Lite::Util::captured {my ($CON) = @_; %s}}, $$call_ref;
   }
 
   sub gen_call {
@@ -401,7 +329,7 @@ use YATT::Lite::Constants;
     my $add_arg = sub {
       my ($name) = @_;
       my $formal = $widget->{arg_dict}{$name} or do {
-	die $self->generror(q{Unknown arg '%s' in widget <%s>}, $name, $wname);
+	die $self->generror(q{Unknown arg '%s' in widget %s}, $name, $wname);
       };
       if (defined $argOrder[my $argno = $formal->argno]) {
 	die $self->generror(q{Duplicate arg '%s'}, $name);
@@ -415,21 +343,9 @@ use YATT::Lite::Constants;
       $self->sync_curline($_->[NODE_LNO]), ", ", $self->add_curline(do {
 	my $name = argName($_);
 	unless (defined $name) {
-	  defined($name = $widget->{arg_order}[$posArgs //= 0])
-	    or die $self->generror("Too many arguments for widget <%s>"); # This may not be called.
-
-          # Positional arguments should not be treated as body argument.
-          if ($widget->{arg_dict}{$name}->is_body_argument) {
-            die $self->generror(
-              "Too many arguments for widget <%s>: %s", $wname
-              , $self->{curtmpl}->source_region($primary->[$posArgs][NODE_BEGIN],
-                                                $primary->[$#$primary][NODE_END])
-            );
-          }
-
-          $posArgs++;
+	  defined($name = $widget->{arg_order}[$posArgs++])
+	    or die $self->generror("Too many args");
 	}
-
 	my $formal = $add_arg->($name);
 	unless (my $passThruVar = passThruVar($_)) {
 	  $self->as_cast_to($formal, argValue($_));
@@ -444,7 +360,7 @@ use YATT::Lite::Constants;
 	  # フラグ立てとして扱って良い型の場合。
 	  $v;
 	} else {
-	  die $self->generror(q{argument '%s' requires value expression like '=...'}, $passThruVar);
+	  die $self->generror(q{valueless arg '%s'}, $passThruVar);
 	}
       });
     } @$primary;
@@ -495,9 +411,6 @@ use YATT::Lite::Constants;
   sub as_varcall_delegate {
     (my MY $self, my ($var, $node)) = @_;
     my Widget $delegate = $var->widget;
-    unless (defined $delegate) {
-      Carp::croak "delegate target widget is empty!";
-    }
     $self->ensure_generated(perl => my Template $tmpl = $delegate->{cf_folder});
     my $that = $tmpl == $self->{curtmpl} ? '$this' : $tmpl->{cf_entns};
     \ sprintf(q{%s->render_%s($CON, %s)}
@@ -550,9 +463,6 @@ use YATT::Lite::Constants;
   }
   sub as_cast_to_code {
     (my MY $self, my ($var, $value)) = @_;
-    unless (ref $value eq 'ARRAY') {
-      die $self->generror(q{Invalid text expression for variable '%s': %s}, $var->varname, $value);
-    }
     local $self->{curtoks} = [@$value];
     my Widget $virtual = $var->widget;
     local $self->{scope} = $self->mkscope
@@ -565,9 +475,7 @@ use YATT::Lite::Constants;
   sub argName  {
     my ($arg, $skip) = @_;
     my $name = $$arg[NODE_PATH];
-    if (array_of_array($name)) {
-      Carp::croak("namelist is given for argName()!");
-    } elsif (not (wantarray and ref $name)) {
+    unless (wantarray and ref $name) {
       $name;
     } elsif (defined $skip) {
       @{$name}[$skip .. $#$name];
@@ -708,8 +616,7 @@ use YATT::Lite::Constants;
     (my MY $self, my ($escape_now)) = splice @_, 0, 2;
     return '' unless @_;
     local $self->{needs_escaping} = 0;
-    if (@_ == 1 and ($_[0][0] eq 'call'
-		       or $_[0][0] eq 'var' and $self->{cf_prefer_call_for_entity})
+    if (@_ == 1 and $_[0][0] eq 'call'
 	and my $macro = $self->can("entmacro_$_[0][1]")) {
       return $macro->($self, $_[0]);
     }
@@ -731,25 +638,6 @@ use YATT::Lite::Constants;
       sprintf(q{YATT::Lite::Util::escape(%s)}, $result);
     }
   }
-  # XXX: partial logic dup with ensure_entity_is_declared
-  sub find_entity {
-    (my MY $self, my ($name)) = @_;
-    my Template $tmpl = $self->{curtmpl};
-    $tmpl->{Item}{"entity\0$name"}
-      // $tmpl->{cf_entns}->can("entity_$name");
-  }
-  sub ensure_entity_is_declared {
-    (my MY $self, my ($name)) = @_;
-    my Template $tmpl = $self->{curtmpl};
-    if ($tmpl->{Item}{"entity\0$name"}) {
-      # Found embedded entity definition.
-      return;
-    }
-    unless ($tmpl->{cf_entns}->can("entity_$name")) {
-      die $self->generror(q!No such entity in namespace "%s": %s!
-			  , $tmpl->{cf_entns}, $name);
-    }
-  }
   sub gen_entlist {
     (my MY $self, my ($escape_now)) = splice @_, 0, 2;
     my @list = map {
@@ -759,16 +647,12 @@ use YATT::Lite::Constants;
   }
   sub as_expr_var {
     (my MY $self, my ($esc_later, $name)) = @_;
-    if (my $var = $self->find_var($name)) {
-      if (my $sub = $self->can("as_expr_var_" . $var->type->[0])) {
-	$sub->($self, $esc_later, $var, $name);
-      } else {
-	$self->as_lvalue($var);
-      }
-    } elsif ($self->find_entity($name)) {
-      $self->gen_entcall($name);
+    my $var = $self->find_var($name)
+      or die $self->generror(q{No such variable '%s'}, $name);
+    if (my $sub = $self->can("as_expr_var_" . $var->type->[0])) {
+      $sub->($self, $esc_later, $var, $name);
     } else {
-      die $self->generror(q{No such variable '%s'}, $name);
+      $self->as_lvalue($var);
     }
   }
   sub as_expr_var_html {
@@ -793,14 +677,13 @@ use YATT::Lite::Constants;
       return $self->as_expr_call_var($var, $name, @_);
     }
 
-    $self->ensure_entity_is_declared($name);
-
-    $self->gen_entcall($name, @_);
-  }
-  sub gen_entcall {
-    (my MY $self, my ($name, @rest)) = @_;
+    my Template $tmpl = $self->{curtmpl};
+    unless ($tmpl->{cf_entns}->can("entity_$name")) {
+      die $self->generror(q!No such entity in namespace "%s": %s!
+			  , $tmpl->{cf_entns}, $name);
+    }
     my $call = sprintf '$this->entity_%s(%s)', $name
-      , scalar $self->gen_entlist(undef, @rest);
+      , scalar $self->gen_entlist(undef, @_);
     $call;
   }
   sub as_expr_call_var {
@@ -847,9 +730,7 @@ use YATT::Lite::Constants;
   }
   sub as_expr_prop {
     (my MY $self, my ($esc_later, $name)) = @_;
-    if ($self->{cf_prefer_call_for_entity}) {
-      $name
-    } elsif ($name =~ /^\w+$/) {
+    if ($name =~ /^\w+$/) {
       "{$name}"
     } else {
       '{'.qtext($name).'}';
@@ -877,16 +758,10 @@ sub feed_arg_spec {
     my ($name, @ext) = argName($arg); # XXX: <yatt:my var:type=value /> は？
     unless (defined $name) {
       $name = $arg_order->[$nth++]
-	or die $trans->generror("Too many args");
+	or die $trans->generror($arg, "Too many args");
     }
     defined (my $argno = $arg_dict->{$name})
-      or die $trans->generror("Unknown arg '%s'", $name);
-
-    if (defined (my $prevNode = $_[$argno])) {
-      die $trans->generror("You may forgot '=' between %s and %s"
-                           , $prevNode->[NODE_PATH]
-                           , $trans->{curtmpl}->node_outer_source($arg));
-    }
+      or die $trans->generror($arg, "Unknown arg '%s'", $name);
 
     $_[$argno] = $arg;
     $found++;
@@ -914,27 +789,22 @@ sub feed_arg_spec {
     # いかん、 cond を生成するなら、body も生成しておかないと、行番号が困る。
 
     foreach my $arg (lexpand($foot)) {
-      (undef, my $kw) = @{$arg->[NODE_PATH]};
-      unless ($kw eq 'else') {
-        die $self->generror("Unknown option for <%s>: %s"
-                            , join(":", @$path), $kw);
+      if ($arg->[NODE_PATH][-1] eq 'else') {
+	$self->feed_arg_spec($arg->[NODE_ATTLIST], \%args, \@args
+			     , my ($if, $unless));
+	my ($fmt, $guard) = do {
+	  if ($if) { (q{elsif (%s) }, $if->[NODE_VALUE]) }
+	  elsif ($unless) { (q{elsif (not %s) }, $unless->[NODE_VALUE]) }
+	  else { (q{else }, undef) }
+	};
+	push @arms, [$fmt, $guard, lexpand($arg->[NODE_VALUE])]
+      } else {
+	push @{$arms[-1]}, lexpand($arg->[NODE_VALUE]);
       }
-      $self->feed_arg_spec($arg->[NODE_ATTLIST], \%args, \@args
-                           , my ($if, $unless));
-      my ($fmt, $guard) = do {
-        if ($if) {
-          (q{elsif (%s) }, $if->[NODE_VALUE]);
-        } elsif ($unless) {
-          (q{elsif (not %s) }, $unless->[NODE_VALUE]);
-        } else {
-          (q{else }, undef);
-        }
-      };
-      push @arms, [$fmt, $guard, lexpand($arg->[NODE_VALUE])]
     }
+    local $self->{scope} = $self->mkscope({}, $self->{scope});
     my @expr = map {
       my ($fmt, $guard, @body) = @$_;
-      local $self->{scope} = $self->mkscope({}, $self->{scope});
       local $self->{curtoks} = [@body];
       (defined $guard
        ? sprintf($fmt, join "", $self->as_list(lexpand($guard))) : $fmt)
@@ -944,23 +814,13 @@ sub feed_arg_spec {
   }
 }
 
-sub is_spread {
-  ref $_[0] eq 'ARRAY'
-    && @{$_[0]} == 4
-    && $_[0][0] eq ''
-    && $_[0][1] eq ''
-    && $_[0][2] eq ''
-}
-sub take_spread_name {
-  $_[0]->[3];
-}
-
 {
   sub macro_my {
     (my MY $self, my $node) = @_;
-    my ($path, $body, $maybeWrappedAttlist, $head, $foot) = nx($node);
+    my ($path, $body, $primary, $head, $foot) = nx($node);
+
     my $has_body = $body && @$body ? 1 : 0;
-    my $simple_adder = sub {
+    my $adder = sub {
       my ($default_type, $arg, $valNode, $skip) = @_;
       my ($name, $typename) = argName($arg, $skip);
       if (my $oldvar = $self->find_var($name)) {
@@ -979,55 +839,9 @@ sub take_spread_name {
 	# typename == source の時が問題だ。
 	my $expr = 'my '.$self->as_lvalue($var);
 	my $value = argValue($valNode);
-	$expr .= defined $value ? (' = '.$self->as_cast_to($var, $value)) : ';';
+	$expr .= $value ? (' = '.$self->as_cast_to($var, $value)) : ';';
       }
     };
-    my $adder = sub {
-      my ($default_type, $arg, $valNode, $skip) = @_;
-      if (array_of_array($arg->[NODE_PATH])) {
-        my (@pre, @main);
-        foreach my $nameSpec (@{$arg->[NODE_PATH]}) {
-          my $is_spread = is_spread($nameSpec->[NODE_PATH]);
-          my ($name, $typename) = do {
-            if ($is_spread) {
-              (take_spread_name($nameSpec->[NODE_PATH]), 'list')
-            } else {
-              argName($nameSpec, $skip);
-            }
-          };
-
-          if (my $oldvar = $self->find_var($name)) {
-            die $self->generror("Conflicting variable '%s'"
-                                ." (previously defined at line %s)"
-                                , $name, $oldvar->lineno // '(unknown)');
-          }
-          $typename ||= $default_type;
-          if (my $sub = $self->can("_macro_my_$typename")) {
-            ...
-          }
-          my $var = $self->{scope}[0]{$name}
-            = $self->mkvar_at(undef, $typename, $name)
-            or die $self->generror("Unknown type '%s' for variable '%s'"
-                                   , $typename, $name);
-
-            if (@pre) {
-              die $self->generror("Multiple spread op is prohibited");
-            }
-          if ($is_spread) {
-            push @pre, 'my '.$self->as_lvalue($var).' = []';
-            push @main, '@'.$self->as_lvalue($var);
-          } else {
-            push @main, 'my '.$self->as_lvalue($var);
-          }
-        }
-        my $main = sprintf(q|(%s) = (%s)|, join(", ", @main)
-                           , join("", $self->as_list(lexpand(argValue($valNode)))));
-        (@pre, $main);
-      } else {
-        $simple_adder->(@_)
-      }
-    };
-    my $primary = $self->node_unwrap_attlist($maybeWrappedAttlist);
     my @assign;
     foreach my $arg (@{$primary}[0 .. $#$primary-$has_body]) {
       push @assign, $adder->(text => $arg, $arg);
@@ -1035,8 +849,8 @@ sub take_spread_name {
     if ($has_body) {
       my $arg = $primary->[-1];
       # XXX: ここは統合できるはず。ただし、NESTED の時に name が無いことを確認すべき。
-      if ($arg->[NODE_TYPE] == TYPE_ATT_NESTED) {
-	foreach my $each (@{$arg->[NODE_BODY]}) {
+      if ($$arg[NODE_TYPE] == TYPE_ATT_NESTED) {
+	foreach my $each (nx($arg, 1)) {
 	  push @assign, $adder->(html => $each, $body);
 	}
       } else {
@@ -1064,14 +878,9 @@ sub take_spread_name {
 
   sub macro_block {
     (my MY $self, my $node) = @_;
+    local $self->{scope} = $self->mkscope({}, $self->{scope});
     my ($path, $body, $primary, $head, $foot) = nx($node);
-    $self->macro_scoped_block_of_tokens(+{}, @{argValue($body)});
-  }
-
-  sub macro_scoped_block_of_tokens {
-    (my MY $self, my ($scope, @tokens)) = @_;
-    local $self->{scope} = $self->mkscope($scope, $self->{scope});
-    local $self->{curtoks} = \@tokens;
+    local $self->{curtoks} = [@{argValue($body)}];
     \ ('{'.$self->as_print('}'));
   }
 }
@@ -1083,7 +892,7 @@ sub take_spread_name {
     my ($path, $body, $primary, $head, $foot) = nx($node);
     $self->feed_arg_spec($primary, \%args, \@args
 			 , my ($list, $my, $nth))
-      or die $self->generror("Not enough arguments for <yatt:foreach>!");
+      or die $self->generror("Not enough arguments!");
 
     my ($prologue, $continue, $epilogue) = ('', '', '');
 
@@ -1132,31 +941,13 @@ sub take_spread_name {
 	}
 	'@'.$self->as_lvalue($found_var);
       } else {
-	die die $self->generror("Unknown variable for foreach list: '%s'", $passThruVarName);
+	die $self->generror("Unknown list=");
       }
     };
 
     local $self->{curtoks} = [@{argValue($body)}];
-
-    # <yatt:foreach ..>\n
-    #                  ↑This newline should be removed.
-    my $statements = "{";
-    if (my $nl = $self->cut_next_nl) {
-      $statements .= $nl;
-    }
-
-    # ___</yatt:foreach>
-    # ↑ This indent is messy too.
-    if (@{$self->{curtoks}} and $self->{curtoks}[-1] =~ /^[\ \t]+\z/) {
-      pop @{$self->{curtoks}};
-    }
-
-    #    ...\n         ← not to remove this newline.
-    # </yatt:foreach>
-    local $self->{no_last_newline} = 0;
-
     local $self->{scope} = $self->mkscope(\%local, $self->{scope});
-    $statements .= $self->as_print('}');
+    my $statements = '{'.$self->as_print('}');
 
     if ($opts and $opts->{fragment}) {
       ($fmt, $loopvar, $listexpr, $statements
@@ -1168,112 +959,9 @@ sub take_spread_name {
   }
 }
 
-{
-  MY->make_arg_spec(\ my %args, \ my @args, qw(if unless local));
-  sub macro_return {
-    (my MY $self, my $node) = @_;
-    my ($path, $body, $primary, $head, $foot) = nx($node);
-
-    if ($foot || $head) {
-      die $self->generror("Unsupported syntax: foot");
-    }
-
-    $self->feed_arg_spec($primary, \%args, \@args
-                         , my ($if, $unless, $local));
-
-    my ($fmt, $guard) = do {
-      if ($if || $unless) {
-        # conditional return
-        # if ()
-        my ($kw, $cond) = do {
-          if ($if) { (if => $if) }
-          elsif ($unless) { (unless => $unless) }
-          else { die "??" }
-        };
-        ("$kw (%s) ", $cond->[NODE_VALUE]);
-      } else {
-        ();
-      }
-    };
-
-    my ($begin, $end) = do {
-      if ($local) {
-        ('{', ' return}');
-      } else {
-        ('{YATT::Lite::Util::rewind($CON); ', ' die \"DONE"}');
-      }
-    };
-
-    my $expr = do {
-      local $self->{scope} = $self->mkscope({}, $self->{scope});
-      local $self->{curtoks} = [lexpand($body->[NODE_VALUE])];
-      (defined $guard
-       ? sprintf($fmt, join "", $self->as_list(lexpand($guard))) : '')
-	.$begin.$self->cut_next_nl.$self->as_print($end);
-    };
-
-    \ $expr;
-  }
-}
-
-# A skeleton for new macro.
-# {
-#   MY->make_arg_spec(\ my %args, \ my @args, qw(if unless));
-#   sub macro_return {
-#     (my MY $self, my $node) = @_;
-#     my ($path, $body, $primary, $head, $foot) = nx($node);
-#
-#     \ ($self->dump_as_comment_line($self->node_as_hash($node)));
-#   }
-# }
-
-sub dump_as_comment_line {
-  (my MY $self, my $node) = @_;
-  my $str = terse_dump($node);
-  $str =~ s/^/# /mg;
-  $str."\n";
-}
-
-# A skelton for new entmacro
-# sub entmacro_XXX {
-#   (my MY $self, my $node) = @_;
-#   sprintf(q{YATT::Lite::Util::escape(%s)}, terse_dump($node));
-# }
-
-sub entmacro_scalar {
-  (my MY $self, my $node) = @_;
-  my (@expr) = $self->gen_entlist(undef, entx($node));
-  sprintf q|scalar(%s)|, join "", @expr;
-}
-
-sub entmacro_not {
-  (my MY $self, my $node) = @_;
-  my (@expr) = $self->gen_entlist(undef, entx($node));
-  sprintf q|(not(%s))|, join "", @expr;
-}
-
-sub entmacro_and {
-  (my MY $self, my $node) = @_;
-  my (@expr) = $self->gen_entlist(undef, entx($node));
-  "(".join(" and ", map {"($_)"} @expr).")";
-}
-
-sub entmacro_or {
-  (my MY $self, my $node) = @_;
-  my (@expr) = $self->gen_entlist(undef, entx($node));
-  "(".join(" or ", map {"($_)"} @expr).")";
-}
-
-sub entmacro_undef {
-  (my MY $self, my $node) = @_;
-  my (@expr) = $self->gen_entlist(undef, entx($node));
-  sprintf q|(undef(%s))|, join "", @expr;
-}
-
-sub entmacro_with_ignoring_die {
-  (my MY $self, my $node) = @_;
-  my (@expr) = $self->gen_entlist(undef, entx($node));
-  sprintf q|($this->YATT->with_ignoring_die(sub {%s}))|, join "", @expr;
+sub entx {
+  my ($node) = @_;
+  @{$node}[2..$#$node];
 }
 
 sub entmacro_if {
@@ -1336,22 +1024,6 @@ sub entmacro_dispatch_one {
     , $prefix, $nargs, join(", ", @list);
 }
 
-sub entmacro___WIDGET__ {
-  (my MY $self, my $node) = @_;
-  my Widget $widget = $self->{curwidget};
-  qtext($widget->{cf_name});
-}
-
-sub entmacro_show_expr {
-  (my MY $self, my $node) = @_;
-  require YATT::Lite::LRXML::FormatEntpath;
-  my (@pipe) = entx($node);
-  \ sprintf q{$this->YATT->render_into($CON, %s, [%s, %s])}
-    , qtext('show_expr')
-    , qtext(YATT::Lite::LRXML::FormatEntpath::format_entpath(@pipe))
-    , join(", ", $self->gen_entpath(undef, @pipe));
-}
-
 use YATT::Lite::Breakpoint qw(break_load_cgen break_cgen);
 break_load_cgen();
 
