@@ -14,9 +14,9 @@ use YATT::Lite::MFields qw/cf_namespace cf_debug_cgen cf_no_lineinfo cf_check_li
 	      cf_die_in_error cf_error_handler
 	      cf_special_entities
 	      cf_lcmsg_sink
-              cf_match_argsroute_first
 
 	      n_compiles
+	      cgen_class
 	    /;
 use YATT::Lite::Util;
 use YATT::Lite::Constants;
@@ -47,7 +47,6 @@ use YATT::Lite::Breakpoint ();
 
      , [Template => -base => MY->File
 	, -alias => 'vfs_file'
-	, -constants => [[can_generate_code => 1]]
 	, -fields => [qw(product parse_ok cf_mtime cf_utf8 cf_age
 			 cf_usage cf_constants
 			 cf_ignore_trailing_newlines
@@ -60,30 +59,7 @@ use YATT::Lite::Breakpoint ();
 				       )]]
     );
 
-  sub YATT::Lite::Core::Part::public_name {
-    (my Part $part) = @_;
-    $part->{cf_name};
-  }
-  sub YATT::Lite::Core::Action::public_name {
-    (my Action $action) = @_;
-    substr($action->{cf_name}, 3);
-  }
-  sub YATT::Lite::Core::Part::method_name {...}
-  sub YATT::Lite::Core::Widget::method_name {
-    (my Widget $widget) = @_;
-    "render_$widget->{cf_name}";
-  }
-  sub YATT::Lite::Core::Action::method_name {
-    (my Action $action) = @_;
-    $action->{cf_name};
-  }
-
-  sub YATT::Lite::Core::Part::configure_folder {
-    (my Part $part, my Folder $folder) = @_;
-    Scalar::Util::weaken($part->{cf_folder} = $folder);
-    # die "Can't weaken!" unless Scalar::Util::isweak($part->{cf_folder});
-  }
-
+  # folder の weaken は parser がしてる。
 #  sub YATT::Lite::Core::Part::source {
 #    (my Part $part) = @_;
 #    join "", map {ref $_ ? "\n" x $$_[0] : $_} @{$part->{source}};
@@ -126,15 +102,13 @@ use YATT::Lite::Breakpoint ();
   }
 
   sub YATT::Lite::Core::Part::reorder_hash_params {
-    (my Widget $widget, my ($orig_params)) = @_;
-    my $params = +{%$orig_params};
+    (my Widget $widget, my ($params)) = @_;
     my @params;
     foreach my $name (map($_ ? @$_ : (), $widget->{arg_order})) {
       push @params, delete $params->{$name};
     }
     if (keys %$params) {
-      die "Unknown args for $widget->{cf_name}: " . join(", ", keys %$params)
-	. "\n";
+      die "Unknown args for $widget->{cf_name}: " . join(", ", keys %$params);
     }
     wantarray ? @params : \@params;
   }
@@ -144,9 +118,8 @@ use YATT::Lite::Breakpoint ();
     $list ||= [];
     foreach my $name ($cgi->param) {
       next unless $name =~ /^[a-z]\w*$/i;
-      my $wname = $widget->{cf_name} ? " for widget '$widget->{cf_name}'" : "";
       my $argdecl = $widget->{arg_dict}{$name}
-	or die "Unknown args$wname: $name\n";
+	or die "Unknown args for widget '$widget->{cf_name}': $name";
       my @value = $cgi->multi_param($name);
       $list->[$argdecl->argno] = $argdecl->type->[0] eq 'list'
 	? \@value : $value[0];
@@ -198,18 +171,20 @@ sub declare_base {
     nonempty(my $fn = $vfs->node_value($att))
       or $vfs->synerror($state, q{base spec is empty!});
 
+    my Folder $dirobj = $tmpl->dirobj;
+
     if ($vfs->{on_memory}) {
       my $o = $vfs->find_file($fn)
 	or $vfs->synerror($state, q{No such base path: %s}, $fn);
       push @$base, $o;
     } else {
-      defined(my $realfn = $vfs->resolve_path_from($tmpl, $fn))
+      defined(my $realfn = $vfs->resolve_path_from($dirobj, $fn))
 	or $vfs->synerror($state, q{Can't find object path for: %s}, $fn);
 
       -e $realfn
 	or $vfs->synerror($state, q{No such base path: %s}, $realfn);
-
-      push @$base, $vfs->find_neighbor_type(undef, $realfn);
+      my $kind = -d $realfn ? 'dir' : 'file';
+      push @$base, $vfs->create($kind => $realfn, parent => $dirobj);
     }
   }
 }
@@ -245,9 +220,7 @@ sub synerror {
     # $self->{parser} ||=
       $self->Parser->new
 	(vfs => $self, $self->cf_delegate
-	 (qw(namespace special_entities
-             match_argsroute_first
-          )
+	 (qw(namespace special_entities)
 	  , [debug_parser => 'debug']
 	  , [tmpl_encoding => 'encoding']
 	 )
@@ -341,7 +314,7 @@ sub synerror {
       $tmpl = $self->find_file($partName)
 	or ($ignore_error and return)
 	  or croak "No such template file: $partName";
-      $part = $tmpl->{Item}{$itemKey} || $self->find_part_from($tmpl, $itemKey)
+      $part = $tmpl->{Item}{$itemKey}
 	or ($ignore_error and return)
 	  or croak "No such $kind in file $partName: $pureName";
     }
@@ -361,28 +334,6 @@ sub synerror {
   sub _itemKey_page { shift; ($_[0], "render_$_[0]") }
   sub _itemKey_action { shift; ("do_$_[0]") x 2; }
 
-  #
-  # Action name => sub {}
-  #
-  sub add_root_action_handler {
-    (my MY $self, my ($name, $sub, $callinfo)) = @_;
-    my Folder $root = $self->{root};
-
-    my ($callpack, $filename, $lineno) = @$callinfo;
-
-    # XXX: This means do_$A.yatt will conflict with "Action $A" in .htyattrc.pl
-    my $action_name = "do_$name";
-
-    *{globref($root->{cf_entns}, $action_name)} = $sub;
-
-    $root->{Item}{$action_name}
-      = $self->Action->new(name => $action_name, kind => 'action'
-			   , folder => $root
-			   , startln => $lineno
-			 );
-
-  }
-
   sub find_renderer {
     my MY $self = shift;
     my ($part, $sub, $pkg) = $self->find_part_handler(@_)
@@ -393,7 +344,15 @@ sub synerror {
   # DirHandler INST 固有 CGEN_perl の生成
   sub get_cgen_class {
     (my MY $self, my $type) = @_;
-    $self->{cf_facade}->get_cgen_class($type);
+    my $sub = $self->can("cgen_$type")
+      || carp "Unknown product type: $type";
+    $self->{cgen_class}{$type} ||= do {
+      my $cg_base = $sub->();
+      # XXX: ref($facade) が INST 固有に成ってなかったら？
+      my $instpkg = ref($self->{cf_facade})."::CGEN_$type";
+      ckeval(qq|package $instpkg; use base qw($cg_base)|);
+      $instpkg;
+    };
   }
 
   # XXX: Action only コンパイルは？
@@ -503,16 +462,11 @@ sub synerror {
 	printf STDERR " => deleted\n" if DEBUG_REBUILD;
 	return; # XXX: ファイルが消された
       } elsif (defined $tmpl->{cf_mtime} and $tmpl->{cf_mtime} >= $mtime) {
-	if (DEBUG_REBUILD) {
-	  printf STDERR " => use cached. mtime(was=$tmpl->{cf_mtime}"
-	    .", now=$mtime) for tmpl=$tmpl\n";
-	}
+	printf STDERR " => not updated.\n" if DEBUG_REBUILD;
 	$self->refresh_deps_for($tmpl) if $self->{cf_always_refresh_deps};
 	return; # timestamp は、キャッシュと同じかむしろ古い
       }
-      if (DEBUG_REBUILD) {
-	printf STDERR " => found update. mtime($mtime) for tmpl=$tmpl\n";
-      }
+      printf STDERR " => found update\n" if DEBUG_REBUILD;
       $tmpl->{cf_mtime} = $mtime;
       my $parser = $self->get_parser;
       # decl のみ parse.
@@ -562,6 +516,11 @@ sub synerror {
   }
 }
 
+sub find_template_from_package {
+  (my MY $self, my $pkg) = @_;
+  $self->{pkg2folder}{$pkg};
+}
+
 use YATT::Lite::Breakpoint ();
 YATT::Lite::Breakpoint::break_load_core();
 
