@@ -2,7 +2,7 @@
 # Parsing and Building. part の型を確定させる所まで請け負うことに。
 package YATT::Lite::LRXML; sub MY () {__PACKAGE__}
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 use 5.010; no if $] >= 5.017011, warnings => "experimental";
 
 use base qw(YATT::Lite::VarMaker);
@@ -473,12 +473,9 @@ sub build_data { shift->Data->new(@_) }
 # declare
 sub declare_base {
   (my MY $self, my Template $tmpl, my ($ns, @args)) = @_;
-  my $att = YATT::Lite::Constants::cut_first_att(\@args) or do {
-    die $self->synerror_at($self->{startln}, q{No base arg});
-  };
-  # !yatt:base dir="..."
-  #            PATH BODY
-  push @{$tmpl->{cf_base}}, [@$att[NODE_PATH, NODE_BODY]]; # XXX: 定形？
+
+  $self->{cf_vfs}->declare_base($self, $tmpl, $ns, @args);
+
   undef;
 }
 
