@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 
 # For future Test::FCGI::Mechanize...
 
