@@ -1,6 +1,7 @@
 package YATT::Lite::Util;
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
+use constant DEBUG_LOOKUP_PATH => $ENV{DEBUG_YATT_UTIL_LOOKUP_PATH};
 
 use URI::Escape ();
 use Tie::IxHash;
@@ -238,12 +239,14 @@ require Scalar::Util;
     $want_ext //= '.yatt';
     my $ixfn = $index_name . $want_ext;
     my @dirlist = grep {defined $_ and -d $_} @$dirlist;
+    print STDERR "dirlist" => terse_dump(@dirlist), "\n" if DEBUG_LOOKUP_PATH;
     my $pi = $path_info;
     my ($loc, $cur, $ext) = ("", "");
   DIG:
     while ($pi =~ s{^/+([^/]+)}{}) {
       $cur = $1;
       $ext = ($cur =~ s/(\.[^\.]+)$// ? $1 : undef);
+      print STDERR terse_dump(cur => $cur, ext => $ext), "\n" if DEBUG_LOOKUP_PATH;
       foreach my $dir (@dirlist) {
 	my $base = "$dir$loc/$cur";
 	if (defined $ext and -r "$base$ext") {
@@ -265,8 +268,10 @@ require Scalar::Util;
       }
     } continue {
       $loc .= "/$cur";
+      print STDERR terse_dump(continuing => $loc), "\n" if DEBUG_LOOKUP_PATH;
       @dirlist = grep {defined} @dirlist;
     }
+      print STDERR terse_dump('end_of_loop'), "\n" if DEBUG_LOOKUP_PATH;
 
     return unless $pi =~ m{^/+$};
 
@@ -275,6 +280,7 @@ require Scalar::Util;
       return ($dir, "$loc/", "$ixfn", "", 1);
     }
 
+      print STDERR terse_dump('at_last'), "\n" if DEBUG_LOOKUP_PATH;
     return;
   }
 
@@ -339,6 +345,9 @@ sub dofile_in {
 
 sub compile_file_in {
   my ($pkg, $file) = @_;
+  if (-d $file) {
+    croak "file '$file' is a directory!";
+  }
   my $sub = dofile_in($pkg, $file);
   unless (defined $sub and ref $sub eq 'CODE') {
     die "file '$file' should return CODE (but not)!\n";
@@ -370,7 +379,7 @@ BEGIN {
 	  $copy;
 	} elsif (ref $str eq 'SCALAR') {
 	  # PASS Thru. (Already escaped)
-	  $$str;
+	  $$str // $ESCAPE_UNDEF; # fail safe
 	} elsif (_is_escapable($str)) {
 	  $str->as_escaped;
 	} elsif (my $sub = UNIVERSAL::can($str, 'cf_pairs')) {
@@ -663,12 +672,6 @@ sub list_isa {
   } @$isa;
 }
 
-sub get_locale_encoding {
-  require Encode;
-  require encoding;
-  Encode::find_encoding(encoding::_get_locale_encoding())->name;
-}
-
 sub set_inc {
   my ($pkg, $val) = @_;
   $pkg =~ s|::|/|g;
@@ -760,13 +763,21 @@ sub parse_nested_query {
   return {} unless defined $_[0] and $_[0] ne '';
   my ($enc) = $_[1];
   my $params = $_[2] // ixhash();
-  foreach my $p (split /[;&]/, $_[0]) {
-    my ($k, $v) = map {
-      s/\+/ /g;
-      my $raw = URI::Escape::uri_unescape($_);
-      $enc ? Encode::decode($enc, $raw) : $raw;
-    } split /=/, $p, 2;
-    normalize_params($params, $k, $v) if defined $k;
+  if (ref $_[0]) {
+    my @pairs = map {$enc ? map(Encode::decode($enc, $_), @$_) : @$_}
+      ref $_[0] eq 'ARRAY' ? $_[0] : [%{$_[0]}];
+    while (my ($k, $v) = splice @pairs, 0, 2) {
+      normalize_params($params, $k, $v);
+    }
+  } else {
+    foreach my $p (split /[;&]/, $_[0]) {
+      my ($k, $v) = map {
+	s/\+/ /g;
+	my $raw = URI::Escape::uri_unescape($_);
+	$enc ? Encode::decode($enc, $raw) : $raw;
+      } split /=/, $p, 2;
+      normalize_params($params, $k, $v) if defined $k;
+    }
   }
   $params;
 }
