@@ -23,9 +23,7 @@ sub add_slot_to {
 BEGIN {
   our @fields = qw(type
 		   varname argno
-		   lineno quote dflag default
-                   from_route
-                );
+		   lineno quote dflag default);
   my $slotNum = 0;
   foreach my $name (@fields) {
     # accessor
