@@ -7,7 +7,7 @@ use base qw(YATT::Lite::Connection);
 use YATT::Lite::MFields
 (qw/cf_cgi
     cf_is_psgi cf_hmv
-    cf_parameters
+    params_hash
 
     cf_site_prefix
 
@@ -17,7 +17,7 @@ use YATT::Lite::MFields
 
     current_user
    /);
-use YATT::Lite::Util qw(globref url_encode nonempty empty rootname lexpand);
+use YATT::Lite::Util qw(globref url_encode nonempty lexpand);
 use YATT::Lite::PSGIEnv;
 
 use YATT::Lite::Util::CGICompat;
@@ -85,7 +85,7 @@ BEGIN {
 
 sub param {
   my PROP $prop = (my $glob = shift)->prop;
-  if (my $ixh = $prop->{cf_parameters}) {
+  if (my $ixh = $prop->{params_hash}) {
     return keys %$ixh unless @_;
     defined (my $key = shift)
       or croak "undefined key!";
@@ -96,7 +96,7 @@ sub param {
 	$ixh->{$key} = shift;
       }
     } else {
-      # If cf_parameters is enabled, value is returned AS-IS.
+      # If params_hash is enabled, value is returned AS-IS.
       $ixh->{$key};
     }
   } elsif (my $hmv = $prop->{cf_hmv}) {
@@ -117,11 +117,11 @@ sub param {
 # Annoying multi_param support.
 sub multi_param {
   my PROP $prop = (my $glob = shift)->prop;
-  if (my $ixh = $prop->{cf_parameters}) {
+  if (my $ixh = $prop->{params_hash}) {
     return keys %$ixh unless @_;
     defined (my $key = shift)
       or croak "undefined key!";
-    # If cf_parameters is enabled, value is returned AS-IS.
+    # If params_hash is enabled, value is returned AS-IS.
     $ixh->{$key};
 
   } elsif (my $hmv = ($prop->{cf_hmv} // do {
@@ -138,31 +138,7 @@ sub multi_param {
 
 sub queryobj {
   my PROP $prop = (my $glob = shift)->prop;
-  $prop->{cf_parameters} || $prop->{cf_hmv} || $prop->{cf_cgi};
-}
-
-*remove_param = *delete_param; *remove_param = *delete_param;
-sub delete_param {
-  my PROP $prop = (my $glob = shift)->prop;
-  my ($key) = @_;
-  unless (defined $key) {
-    croak "Undefined key!";
-  }
-  if (my $dict = $prop->{cf_parameters}) {
-    # For nested_query (array_param)
-    delete $dict->{$key};
-  } elsif ($dict = $prop->{cf_hmv}) {
-    # For direct Hash::MultiValue.
-    $dict->remove($key);
-  } elsif (($dict = $prop->{cf_cgi})->can("parameters")) {
-    # For Plack::Request
-    $dict->parameters->remove($key);
-  } elsif ($dict->can("delete")) {
-    # For CGI family.
-    $dict->delete($key);
-  } else {
-    croak "No queryobj found!";
-  }
+  $prop->{params_hash} || $prop->{cf_hmv} || $prop->{cf_cgi};
 }
 
 #========================================
@@ -171,7 +147,6 @@ sub configure_cgi {
   my PROP $prop = (my $glob = shift)->prop;
   $prop->{cf_cgi} = my $cgi = shift;
   return unless $glob->is_form_content_type($cgi->content_type);
-  return if $prop->{cf_parameters};
   unless ($prop->{cf_no_nested_query}) {
     if ($prop->{cf_is_psgi}) {
       $glob->convert_array_param_psgi($cgi);
@@ -208,7 +183,7 @@ sub convert_array_param_psgi {
   my PROP $prop = (my $glob = shift)->prop;
   my ($req) = @_;
   my Env $env = $prop->{cf_env};
-  $prop->{cf_parameters} = do {
+  $prop->{params_hash} = do {
     if ($env->{CONTENT_TYPE} and defined $env->{CONTENT_LENGTH}) {
       my $body = $glob->parse_nested_query([$req->body_parameters->flatten]);
       my $qs = $glob->parse_nested_query($env->{QUERY_STRING});
@@ -230,22 +205,19 @@ sub convert_array_param_cgi {
   my PROP $prop = (my $glob = shift)->prop;
   my ($cgi) = @_;
   return if ($cgi->content_type // "") eq "application/json";
-  $prop->{cf_parameters}
+  $prop->{params_hash}
     = $glob->parse_nested_query($cgi->query_string);
 }
 
 # Location(path part of url) of overall SiteApp.
 sub site_location {
-  shift->site_prefix . '/';
+  my PROP $prop = (my $glob = shift)->prop;
+  $prop->{cf_site_prefix} . '/';
 }
 *site_loc = *site_location; *site_loc = *site_location;
 sub site_prefix {
   my PROP $prop = (my $glob = shift)->prop;
-  # Note: This is safe because site_prefix 0 is meaningless(I hope).
-  $prop->{cf_site_prefix} || do {
-    my Env $env = $prop->{cf_env};
-    $env->{'yatt.script_name'} // ''
-  }
+  $prop->{cf_site_prefix};
 }
 
 # Location of DirApp
@@ -254,7 +226,6 @@ sub location {
   (my $loc = ($prop->{cf_location} // '')) =~ s,/*$,/,;
   $loc;
 }
-*dir_location = *location; *dir_location = *location;
 
 sub _invoke_or {
   my ($default, $obj, $method, @args) = @_;
@@ -286,7 +257,7 @@ sub mkurl {
 
   my $path = do {
     if (defined $file and $file =~ m!^/!) {
-      $glob->site_prefix.$file;
+      $prop->{cf_site_prefix}.$file;
     } else {
       my ($orig, $dir) = ('');
       if (($dir = $req) =~ s{([^/]+)$}{}) {
@@ -313,33 +284,16 @@ sub mkurl {
 
 sub mkprefix {
   my PROP $prop = (my $glob = shift)->prop;
-  my Env $env = $prop->{cf_env};
-  my $scheme = $env->{'psgi.url_scheme'} || $prop->{cf_cgi}->protocol;
+  my $scheme = $prop->{cf_env}{'psgi.url_scheme'} || $prop->{cf_cgi}->protocol;
   my $host = $glob->mkhost($scheme);
   $scheme . '://' . $host . join("", @_);
 }
 
-sub http_host_domain {
-  my PROP $prop = (my $glob = shift)->prop;
-  my Env $env = $prop->{cf_env};
-  my $host = $env->{HTTP_HOST}
-    or return undef;
-  $host =~ s/:\d+$//;
-  $host;
-}
-
-sub server_name_or_localhost {
-  my PROP $prop = (my $glob = shift)->prop;
-  my ($default) = @_;
-  my Env $env = $prop->{cf_env};
-  $env->{SERVER_NAME} || ($default // 'localhost');
-}
-
 sub mkhost {
   my PROP $prop = (my $glob = shift)->prop;
   my ($scheme) = @_;
   $scheme ||= 'http';
-  my Env $env = $prop->{cf_env};
+  my $env = $prop->{cf_env};
 
   # XXX? Is this secure?
   return $env->{HTTP_HOST} if nonempty($env->{HTTP_HOST});
@@ -382,9 +336,6 @@ sub mkquery {
 	for $fgetall->($param, $key);
     }
   } elsif (ref $param eq 'ARRAY') {
-    if (grep {not defined} @$param) {
-      croak "Undef found in mkquery()! " . YATT::Lite::Util::terse_dump($param);
-    }
     my @list = @$param;
     while (my ($key, $value) = splice @list, 0, 2) {
       push @enc_param, $self->url_encode($key).'='.$self->url_encode($value);
@@ -398,53 +349,6 @@ sub mkquery {
   }
 }
 
-
-# script_name + path_info - subpage
-# (script_name == location of this dir (DirApp))
-#
-sub file_location {
-  my PROP $prop = (my $glob = shift)->prop;
-  my Env $env = $prop->{cf_env};
-  my $loc = do {
-    if (my $sn = $env->{'yatt.script_name'}) {
-      "$sn/"
-    } else {
-      $prop->{cf_location} // "/";
-    }
-  };
-  if (not $prop->{cf_is_index}
-      and my $fn = $prop->{cf_file}) {
-    $fn =~ s/\..*//;
-    $loc .= $fn;
-  }
-  $loc;
-}
-
-# XXX: not yet tested.
-sub is_current_file {
-  my PROP $prop = (my $glob = shift)->prop;
-  my ($fn) = @_;
-  $glob->file_location eq $fn
-}
-
-sub is_current_page {
-  my PROP $prop = (my $glob = shift)->prop;
-  my ($file, $page) = do {
-    @_ <= 1 ? (rootname($prop->{cf_file}), $_[0]) : @_;
-  };
-  rootname($prop->{cf_file}) eq $file
-    or return 0;
-  $page //= '';
-  $page =~ s{^/}{}; # Treat /foo as foo.
-  if (empty(my $subpath = $prop->{cf_subpath})) {
-    $page eq '';
-  } elsif ($page eq '') {
-    0
-  } else {
-    $subpath =~ m{^/$page};
-  }
-}
-
 sub mapped_path {
   my PROP $prop = (my $glob = shift)->prop;
   my @path = do {
@@ -473,8 +377,8 @@ sub request_path {
 
 sub request_uri {
   my PROP $prop = (my $glob = shift)->prop;
-  if (my Env $env = $prop->{cf_env}) {
-    $env->{REQUEST_URI};
+  if ($prop->{cf_env}) {
+    $prop->{cf_env}{REQUEST_URI};
   } elsif ($prop->{cf_cgi}
       and my $sub = $prop->{cf_cgi}->can('request_uri')) {
     $sub->($prop->{cf_cgi});
