@@ -382,10 +382,7 @@ sub ensure_table_populated {
 sub sqlite_begin_create {
   (my MY $schema) = @_;
   # To speedup create statements.
-  my $dbh = $schema->dbh;
-  if ($dbh->{AutoCommit}) {
-    $dbh->do("PRAGMA synchronous = OFF");
-  }
+  $schema->dbh->do("PRAGMA synchronous = OFF");
 }
 
 sub expand_codevalue {
