@@ -1,6 +1,6 @@
 package YATT::Lite::WebMVC0::SiteApp;
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 use Carp;
 use YATT::Lite::Breakpoint;
 sub MY () {__PACKAGE__}
@@ -22,11 +22,14 @@ use YATT::Lite::MFields qw/cf_noheader
 			   cf_debug_backend
 			   cf_psgi_static
 			   cf_psgi_fallback
+			   cf_per_role_docroot
+			   cf_per_role_docroot_key
+			   cf_default_role
 			   cf_backend
 			   cf_site_config
 			   cf_logfile
-			   cf_use_subpath
 			   cf_debug_allowed_ip
+			   cf_overwrite_status_code_for_errors_as
 			   re_handled_ext
 			 /;
 
@@ -47,13 +50,17 @@ use File::Basename;
 sub after_new {
   (my MY $self) = @_;
   $self->SUPER::after_new();
-  $self->{cf_use_subpath} //= 1;
   $self->{re_handled_ext} = qr{\.($self->{cf_ext_public}|ydo)$};
+  $self->{cf_per_role_docroot_key} ||= $self->default_per_role_docroot_key;
+  $self->{cf_default_role} ||= $self->default_default_role;
 }
 
+sub default_per_role_docroot_key { 'yatt.role' }
+sub default_default_role { 'nobody' }
+
 sub _cf_delegates {
   (shift->SUPER::_cf_delegates
-   , qw/use_subpath/);
+   , qw(overwrite_status_code_for_errors_as));
 }
 
 #========================================
@@ -90,6 +97,17 @@ sub get_dirhandler {
   $self->{path2yatt}{$dirPath} ||= $self->load_yatt($dirPath);
 }
 
+sub get_lochandler {
+  (my MY $self, my ($location, $tmpldir)) = @_;
+  if ($self->{cf_per_role_docroot}) {
+    # When per_role_docroot is on, $tmpldir already points
+    # $per_role_docroot/$role. So just append $location.
+    $self->get_dirhandler($tmpldir.$location);
+  } else {
+    $self->SUPER::get_lochandler($location, $tmpldir);
+  }
+}
+
 #----------------------------------------
 # preload_handlers
 
@@ -130,7 +148,8 @@ sub to_app {
 #  unless (defined $self->{cf_app_root}) {
 #    croak "app_root is undef!";
 #  }
-  unless (defined $self->{cf_doc_root}) {
+  unless (defined $self->{cf_doc_root}
+	  or defined $self->{cf_per_role_docroot}) {
     croak "document_root is undef!";
   }
   return $self->SUPER::to_app(@_);
@@ -163,15 +182,36 @@ sub call {
     return $self->psgi_error(403, "Forbidden $deny");
   }
 
+  if (not $self->{cf_no_unicode_params}
+      and $self->{cf_output_encoding}) {
+    $env->{PATH_INFO} = Encode::decode($self->{cf_output_encoding}
+				       , $env->{PATH_INFO});
+  }
+
+  if ($self->{loc2psgi_dict}
+      and my $psgi_app = $self->lookup_psgi_mount($env->{PATH_INFO})) {
+    require Plack::Util;
+    return Plack::Util::run_app($psgi_app, $env);
+  }
+
   # XXX: user_dir?
   my ($tmpldir, $loc, $file, $trailer, $is_index)
     = my @pi = $self->split_path_info($env);
 
+  my ($realdir, $virtdir);
+  if (@pi) {
+    $realdir = "$tmpldir$loc";
+    $virtdir = defined $self->{cf_doc_root}
+      ? "$self->{cf_doc_root}$loc" : $realdir;
+  }
+
   if ($self->has_htdebug("path_info")) {
     return $self->psgi_dump([tmpldir   => $tmpldir]
 			    , [loc     => $loc]
 			    , [file    => $file]
-			    , [trailer => $trailer]);
+			    , [trailer => $trailer]
+			    , [virtdir => $virtdir, realdir => $realdir]
+			  );
   }
 
   if ($self->{cf_debug_psgi}) {
@@ -193,10 +233,8 @@ sub call {
     return $self->psgi_handle_fallback($env);
   }
 
-  my $virtdir = "$self->{cf_doc_root}$loc";
-  my $realdir = "$tmpldir$loc";
   unless (-d $realdir) {
-    return $self->psgi_error(404, "Not found: $virtdir");
+    return $self->psgi_error(404, "Not found: $loc");
   }
 
   # Default index file.
@@ -223,6 +261,7 @@ sub call {
   my $req = Plack::Request->new($env);
 
   my @params = (env => $env
+		, path_info => $env->{PATH_INFO}
 		, $self->connection_quad([$virtdir, $loc, $file, $trailer])
 		, $is_index ? (is_index => 1) : ()
 		, is_psgi => 1, cgi => $req);
@@ -285,12 +324,6 @@ sub make_debug_params {
 
 #========================================
 
-sub psgi_file_app {
-  my ($pack, $path) = @_;
-  require Plack::App::File;
-  Plack::App::File->new(root => $path)->to_app;
-}
-
 sub psgi_handle_static {
   (my MY $self, my Env $env) = @_;
   my $app = $self->{cf_psgi_static}
@@ -305,8 +338,9 @@ sub psgi_handle_static {
 
 sub psgi_handle_fallback {
   (my MY $self, my Env $env) = @_;
-  my $app = $self->{cf_psgi_fallback}
-    or return [404, [], ["Cannot understand:", $env->{PATH_INFO}]];
+  (my $app = $self->{cf_psgi_fallback}
+   ||= $self->psgi_file_app($self->{cf_doc_root}))
+    or return [404, [], ["Cannot understand: ", $env->{PATH_INFO}]];
 
   local $env->{PATH_INFO} = $self->trim_site_prefix($env->{PATH_INFO});
 
@@ -338,7 +372,8 @@ sub is_done {
 sub split_path_info {
   (my MY $self, my Env $env) = @_;
 
-  if (nonempty($env->{PATH_TRANSLATED})
+  if (! $self->{cf_per_role_docroot}
+      && nonempty($env->{PATH_TRANSLATED})
       && $self->is_path_translated_mode($env)) {
     #
     # [1] PATH_TRANSLATED mode.
@@ -363,8 +398,19 @@ sub split_path_info {
     #
     # [2] Template lookup mode.
     #
+
+    my $tmpldirs = do {
+      if ($self->{cf_per_role_docroot}) {
+        my $user = $env->{$self->{cf_per_role_docroot_key}};
+        $user ||= $self->{cf_default_role};
+        ["$self->{cf_per_role_docroot}/$user"]
+      } else {
+        $self->{tmpldirs}
+      }
+    };
+
     lookup_path($env->{PATH_INFO}
-		, $self->{tmpldirs}
+		, $tmpldirs
 		, $self->{cf_index_name}, ".$self->{cf_ext_public}"
 		, $self->{cf_use_subpath});
   } else {
@@ -565,6 +611,22 @@ Entity is_debug_allowed_ip => sub {
 				    // ['127.0.0.1']);
 };
 
+foreach my $item (map([lc($_) => uc($_)]
+		      , qw/SCRIPT_NAME
+			   PATH_INFO
+			   REQUEST_URI
+
+			   SCRIPT_URI
+			   SCRIPT_URL
+			   SCRIPT_FILENAME
+			   /)) {
+  my ($method, $env_name) = @$item;
+  Entity $method => sub {
+    my Env $env = $CON->env;
+    $env->{$env_name};
+  };
+}
+
 #========================================
 
 YATT::Lite::Breakpoint::break_load_dispatcher();
