@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 use Jcode qw(getcode);
 
 use Getopt::Long;
